"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.updateStructureIndex = updateStructureIndex;
exports.clearEdgeFromStructureIndex = clearEdgeFromStructureIndex;
exports.clearStructureIndex = clearStructureIndex;
exports.upgradeStructureIndexToMulti = upgradeStructureIndexToMulti;

/**
 * Graphology Indexes Functions
 * =============================
 *
 * Bunch of functions used to compute or clear indexes.
 */

/**
 * Function updating the 'structure' index with the given edge's data.
 * Note that in the case of the multi graph, related edges are stored in a
 * set that is the same for A -> B & B <- A.
 *
 * @param {Graph}    graph      - Target Graph instance.
 * @param {EdgeData} edgeData   - Added edge's data.
 * @param {NodeData} sourceData - Source node's data.
 * @param {NodeData} targetData - Target node's data.
 */
function updateStructureIndex(graph, undirected, edgeData, source, target, sourceData, targetData) {
  var multi = graph.multi;
  var outKey = undirected ? 'undirected' : 'out',
      inKey = undirected ? 'undirected' : 'in'; // Handling source

  var edgeOrSet = sourceData[outKey][target];

  if (typeof edgeOrSet === 'undefined') {
    edgeOrSet = multi ? new Set() : edgeData;
    sourceData[outKey][target] = edgeOrSet;
  }

  if (multi) edgeOrSet.add(edgeData); // If selfLoop, we break here

  if (source === target) return; // Handling target (we won't add the edge because it was already taken
  // care of with source above)

  if (typeof targetData[inKey][source] === 'undefined') targetData[inKey][source] = edgeOrSet;
}
/**
 * Function clearing the 'structure' index data related to the given edge.
 *
 * @param {Graph}    graph    - Target Graph instance.
 * @param {EdgeData} edgeData - Dropped edge's data.
 */


function clearEdgeFromStructureIndex(graph, undirected, edgeData) {
  var multi = graph.multi;
  var sourceData = edgeData.source,
      targetData = edgeData.target;
  var source = sourceData.key,
      target = targetData.key; // NOTE: since the edge set is the same for source & target, we can only
  // affect source

  var outKey = undirected ? 'undirected' : 'out',
      sourceIndex = sourceData[outKey];
  var inKey = undirected ? 'undirected' : 'in';

  if (target in sourceIndex) {
    if (multi) {
      var set = sourceIndex[target];

      if (set.size === 1) {
        delete sourceIndex[target];
        delete targetData[inKey][source];
      } else {
        set["delete"](edgeData);
      }
    } else delete sourceIndex[target];
  }

  if (multi) return;
  var targetIndex = targetData[inKey];
  delete targetIndex[source];
}
/**
 * Function clearing the whole 'structure' index.
 *
 * @param {Graph} graph - Target Graph instance.
 */


function clearStructureIndex(graph) {
  graph._nodes.forEach(function (data) {
    // Clearing now useless properties
    if (typeof data["in"] !== 'undefined') {
      data["in"] = {};
      data.out = {};
    }

    if (typeof data.undirected !== 'undefined') {
      data.undirected = {};
    }
  });
}
/**
 * Function used to upgrade a simple `structure` index to a multi on.
 *
 * @param {Graph}  graph - Target Graph instance.
 */


function upgradeStructureIndexToMulti(graph) {
  graph._nodes.forEach(function (data, node) {
    // Directed
    if (data.out) {
      for (var neighbor in data.out) {
        var edges = new Set();
        edges.add(data.out[neighbor]);
        data.out[neighbor] = edges;
        graph._nodes.get(neighbor)["in"][node] = edges;
      }
    } // Undirected


    if (data.undirected) {
      for (var _neighbor in data.undirected) {
        if (_neighbor > node) continue;

        var _edges = new Set();

        _edges.add(data.undirected[_neighbor]);

        data.undirected[_neighbor] = _edges;
        graph._nodes.get(_neighbor).undirected[node] = _edges;
      }
    }
  });
}